"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Venmo = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
class Venmo {
    constructor(deviceId) {
        this._fetch = (method, path, body, headers, options) => __awaiter(this, void 0, void 0, function* () {
            return (0, node_fetch_1.default)(`${Venmo.base}/${path}`, Object.assign({ method: method, body: JSON.stringify(body), headers: Object.assign({ 'Content-Type': 'application/json' }, headers) }, options));
        });
        this.login = (phoneEmailUsername, password, headers) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this._fetch('POST', 'oauth/access_token', {
                phone_email_or_username: phoneEmailUsername,
                client_id: 1,
                password: password
            }, Object.assign({ 'device-id': this.deviceId }, headers));
            if (res.ok) {
                this.access = (yield res.json());
                return res;
            }
        });
        this.logout = () => __awaiter(this, void 0, void 0, function* () {
            const logoutRes = yield this.request('DELETE', 'oauth/access_token');
            if (logoutRes.ok) {
                this.access = undefined;
            }
        });
        this.twoFactorToken = (otpSecret) => {
            return this._fetch('POST', `account/two-factor/token`, {
                via: "sms"
            }, {
                'device-id': this.deviceId,
                'venmo-otp-secret': otpSecret,
            });
        };
        this.easyLogin = (phoneEmailUsername, password, otpCallback) => __awaiter(this, void 0, void 0, function* () {
            const loginRes = yield this.login(phoneEmailUsername, password);
            if (loginRes === null || loginRes === void 0 ? void 0 : loginRes.ok)
                return this.access;
            const otpSecret = loginRes === null || loginRes === void 0 ? void 0 : loginRes.headers.get('venmo-otp-secret');
            if (!otpSecret) {
                throw new Error('No otp secret');
            }
            const otpRes = yield this.twoFactorToken(otpSecret);
            if (!otpRes.ok) {
                throw new Error('Two factor request failed');
            }
            if (!otpCallback) {
                throw new Error('No otp callback');
            }
            const otpCode = yield otpCallback();
            const otpLogin = yield this.login(phoneEmailUsername, password, {
                'venmo-otp-secret': otpSecret,
                'venmo-otp': otpCode
            });
            if (otpLogin === null || otpLogin === void 0 ? void 0 : otpLogin.ok)
                return this.access;
            throw new Error('Two factor failed. Check code.');
        });
        this.request = (method, path, body) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!((_a = this.access) === null || _a === void 0 ? void 0 : _a.access_token)) {
                throw new Error('Not logged in');
            }
            return this._fetch(method, path, body, {
                'Authorization': `Bearer ${this.access.access_token}`,
            });
        });
        this.requestJson = (method, path, body) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this.request(method, path, body);
            return res.ok ? res.json() : null;
        });
        this.userQuery = (query, limit, offset) => __awaiter(this, void 0, void 0, function* () {
            return this.requestJson('GET', `users?query=${query}&limit=${limit || 10}&offset=${offset || 0}`).then(res => res ? res.data : []);
        });
        this.pay = (userId, amount, note, audience) => __awaiter(this, void 0, void 0, function* () {
            return this.requestJson('POST', 'payments', {
                user_id: userId,
                amount: amount,
                note: note,
                audience: audience ? audience : 'private'
            });
        });
        this.getMyProfile = () => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', 'me'); });
        this.getUserProfile = (userId) => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', `users/${userId}`); });
        this.getUserFriendList = (userId, limit) => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', `users/${userId}/friends?limit=${limit || 10}`); });
        this.getUserTransactions = (userId) => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', `stories/target-or-actor/${userId}`); });
        this.getPaymentMethods = () => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', `payment-methods`); });
        this.getTransaction = (transactionId) => __awaiter(this, void 0, void 0, function* () { return this.requestJson('GET', `stories/${transactionId}`); });
        this.deviceId = deviceId || '88884260-05O3-8U81-58I1-2WA76F357GR9';
        this.access = undefined;
    }
}
exports.Venmo = Venmo;
Venmo.base = 'https://api.venmo.com/v1';
